# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


package Scmbug::Activity;

use strict;
use Scmbug::Activity;
use Scmbug::Common;
use Scmbug::Error;
use Data::Dumper;



# Constructor
sub new {
    my $type = shift;
    my $self = {};
    return bless $self, $type;
}



sub name {
    my $self = shift;
    my $data = shift;
    
    if ( $data ) {
	$self->{ name } = $data;
    } else {
	return $self->{ name };
    }
}



sub user {
    my $self = shift;
    my $data = shift;

    if ( $data ) {
	$self->{ user } = $data;
    } else {
	return $self->{ user };
    }
}



sub ids {
    my $self = shift;
    
    if ( !defined ( $self->{ ids } ) ) {
	$self->{ ids } = [];
    } else {
	return $self->{ ids };
    }
}



sub log_message {
    my $self = shift;
    my $data = shift;
    
    if ( $data ) {
	$self->{ log_message } = $data;
    } else {
	return $self->{ log_message };
    }
}



sub files {
    my $self = shift;
    my $data = shift;
    
    if ( $data ) {
	$self->{ files } = $data;
    } else {
	return $self->{ files };
    }
}



sub original_log_message {
    my $self = shift;
    my $data = shift;
    
    if ( $data ) {
	$self->{ original_log_message } = $data;
    } else {
	return $self->{ original_log_message };
    }
}



sub branch_name {
    my $self = shift;
    my $data = shift;

    if ( $data ) {
	$self->{ branch_name } = $data;
    } else {
	return $self->{ branch_name };
    }
}



sub product_name {
    my $self = shift;
    my $data = shift;
    
    if ( $data ) {
	$self->{ product_name } = $data;
    } else {
	return $self->{ product_name };
    }
}



sub repository {
    my $self = shift;
    my $data = shift;
    
    if ( $data ) {
	$self->{ repository } = $data;
    } else {
	return $self->{ repository };
    }
}



sub product_name_prefixes {
    my $self = shift;
    my $data = shift;
    
    if ( $data ) {
	$self->{ product_name_prefixes } = $data;
    } else {
	return $self->{ product_name_prefixes };
    }
}



#
# If the product_name must be autodetected do so
#
sub prepare_product_name {
    my $self = shift;
    my $must_autodetect_product_name = 0;
    my $product_name;

    if ( !defined ( $self->{ product_name } ) ) {
	$must_autodetect_product_name = 1;
    }

    if ( $must_autodetect_product_name ) {
	# If the product_name must be autodetected, do so for every
	# single file
	foreach my $file_name ( keys %{ $self->{ files } } ) {
	    $product_name = $self->autodetect_product_name( $file_name );
	}
	$self->product_name( $product_name );
    }
}



sub autodetect_product_name {
    my $self = shift;
    my $filename = shift;
    my $product_name_was_detected = 0;
    my $product_name_autodetected = '';
    my $product_name = '';

    # For each prefix in the list, test if the prefix matches the full
    # path of the filename. In that case, grab the first directory
    # name and use that as the product name.
    my @prefixes = split( ",", $self->product_name_prefixes() );
    foreach my $prefix ( @prefixes ) {
	$product_name = $filename;

	if ( $product_name =~ m/$prefix/ ) {
	    $product_name_was_detected = 1;

	    # Set the product_name to be the first directory right
	    # after the prefix.
	    $product_name = $1;

	    # If this is the first time we are detecting, store the
	    # first match
	    if ( $product_name_autodetected eq '' ) {
		$product_name_autodetected = $product_name;
	    }

	    # If this is not the first time we are detecting, it
	    # should match the last match
	    if ( $product_name_autodetected ne $product_name ) {
		scmbug_error( $GLUE_ERROR_AUTODETECTED_MORE_THAN_ONE_PRODUCT_NAME, "More than one product name autodetected. Detected '$product_name' after already detecting '$product_name_autodetected'.\n" );
	    }
	}
    }

    if ( $product_name_was_detected == 0 ){
	scmbug_error( $GLUE_ERROR_CANNOT_AUTODETECT_PRODUCT_NAME, "Unable to autodetect product name for filename '$filename'.\n" );
    }

    return $product_name_autodetected;
}



1;
